import "Turbine";
import "Turbine.Gameplay";

Settings = class();

function Settings:Constructor()
	self:Load();
end

function Settings:Load()
	self.settingsTable = Turbine.PluginData.Load(Turbine.DataScope.Account,"ChallengeTracker");
	self.currentPlayer = Turbine.Gameplay.LocalPlayer:GetInstance():GetName();
	
	if(self.settingsTable == nil) then
		self.settingsTable = {};
	end
	if(self.settingsTable.resetSchedule == nil) then
		self.settingsTable.resetSchedule = {};
	end
	if(self.settingsTable.players == nil) then
		self.settingsTable.players = {};
		self.settingsTable.players.length = 0;
	end
	if(self.settingsTable.locks == nil) then
		self.settingsTable.locks = {};
	end
	if(self.settingsTable.instances == nil) then
		self.settingsTable.instances = {};
		self.settingsTable.instanceQuestMap = {};
		self.settingsTable.instances.length = 0;
		self:LoadInstances();
	end
	if(self.settingsTable.weeklyResetTime == nil) then
		self:GetNewWeeklyResetTime();
	end
	if(self.settingsTable.biWeeklyResetTime == nil) then
		self:GetNewBiWeeklyResetTime();
	end		
	if(self:IsTimeToWeeklyReset()) then
		self:ResetWeekly();
	elseif(self:IsTimeToBiWeeklyReset()) then
		self:ResetBiWeekly();
	end
	
	self:AddPlayer(self.currentPlayer);
end

function Settings:IsTimeToWeeklyReset()
	local date = Turbine.Engine.GetDate();
	
	local d1 = self.settingsTable.weeklyResetTime.DayOfYear;
	local d2 = (date.Year - self.settingsTable.weeklyResetTime.Year)*365 + date.DayOfYear;
	
	if(d2-d1 > 7) then return true; end
	
	local d3 = self.settingsTable.weeklyResetTime.DayOfWeek;
	local d4 = date.DayOfWeek;
	if(d4 < d3) then d4 = d4 + 7; end
	local h5 = self.settingsTable.weeklyResetTime.Hour;
	local h6 = date.Hour;
	
	local b7 = (d3 < 5) or ((d3 == 5) and (h5 < 6));
	local b8 = (d4 > 5) or ((d4 == 5) and (h6 >= 6));
	
	if(b7 and b8) then return true; end
	
	return false;
end

function Settings:IsTimeToBiWeeklyReset()
	local date = Turbine.Engine.GetDate();
	
	local d1 = self.settingsTable.biWeeklyResetTime.DayOfYear;
	local d2 = (date.Year - self.settingsTable.biWeeklyResetTime.Year)*365 + date.DayOfYear;
	
	if(d2-d1 > 4) then return true; end
	
	local d3 = self.settingsTable.biWeeklyResetTime.DayOfWeek;
	local d4 = date.DayOfWeek;
	if(d4 < d3) then d4 = d4 + 7; end
	local h5 = self.settingsTable.biWeeklyResetTime.Hour;
	local h6 = date.Hour;
	
	local b7 = (d3 < 5) or ((d3 == 5) and (h5 < 6));
	local b8 = (d4 > 5) or ((d4 == 5) and (h6 >= 6));
	local b9 = (d3 < 2) or ((d3 == 2) and (h5 < 6));
	local b10 = (d4 > 2) or ((d4 == 5) and (h6 >= 6)); 
	
	if((b7 and b8) or (b9 and b10)) then return true; end
	
	return false;
end

function Settings:LoadInstances()
--	self:AddInstance("The Tower of Orthanc: Fire/Frost","Tower of Orthanc: Rings of Fire and Frost","Weekly");
--	self:AddInstance("The Tower of Orthanc: Lightning","Tower of Orthanc: Ring of Lightning","Weekly");
--	self:AddInstance("The Tower of Orthanc: Acid","Tower of Orthanc: Ring of Acid","Weekly");
--	self:AddInstance("The Tower of Orthanc: Shadow","Tower of Orthanc: Ring of Shadow","Weekly");
--	self:AddInstance("The Tower of Orthanc: Saruman","Tower of Orthanc: Saruman's Ring","Weekly");
--	self:AddInstance("The Foundry","Challenge: The Ring-forge Falls","Bi-Weekly");
--	self:AddInstance("Dargn\195\161kh Unleased","Challenge: Dargn\195\161kh Unleashed","Bi-Weekly");
--	self:AddInstance("Pits of Isengard","Challenge: Pits of Isengard","Bi-Weekly");
--	self:AddInstance("Fangorn's Edge","Challenge: Fangorn's Savior","Bi-Weekly");
--	self:AddInstance("Draigoch's Lair","Draigoch's Lair","Weekly");
--	self:AddInstance("Helegrod: Thorog","Helegrod Drugoth - Raid","Weekly");
--	self:AddInstance("Helegrod: Drake Wing","Helegrod Drake - Raid","Weekly");
--	self:AddInstance("Helegrod: Giant Wing","Helegrod Giant - Raid","Weekly");
--	self:AddInstance("Helegrod: Spider Wing","Helegrod Spider - Raid","Weekly");
--	self:AddInstance("Ford of Bruinen","Ford of Bruinen - Raid","Weekly");
--	self:AddInstance("Siege of Gondamon","Siege of Gondamon - Raid","Weekly");
--	self:AddInstance("Trouble in Tuckborough","Trouble in Tuckborough - Raid","Weekly");
--	self:AddInstance("Stand at Amon S\195\186l","Stand at Amon S\195\186l - Raid","Weekly");
--	self:AddInstance("Strike Against Dannenglor","Strike Against Dannenglor - Raid","Weekly");
--	self:AddInstance("Protectors of Thang\195\186lhad","Protectors of Thang\195\186lhad - Raid","Weekly");
--	self:AddInstance("Breaching the Necromancer's Gate","Breaching the Necromancer's Gate - Raid","Weekly");
--	self:AddInstance("Assault on the Ringwraiths' Lair","Assault on the Ringwraiths' Lair - Raid","Weekly");
--	self:AddInstance("The Battle in the Tower","The Battle in the Tower - Raid","Weekly");
--	self:AddInstance("Survival: Barrow-Downs","Survival: Barrow-Downs - Raid","Weekly");
--	self:AddInstance("Defence of the Prancing Pony","Defence of the Prancing Pony - Raid","Weekly");
--	self:AddInstance("Thievery and Mischief","Thievery and Mischief - Raid","Weekly");
--	self:AddInstance("Rescue in N\195\186rz Gh\195\161shu","Rescue in N\195\186rz Gh\195\161shu - Raid","Weekly");
--	self:AddInstance("Battle of the Deep-way","Battle of the Deep-way - Raid","Weekly");
--	self:AddInstance("Battle of the Way of Smiths","Battle of the Way of Smiths - Raid","Weekly");
--	self:AddInstance("Battle of the Twenty-first Hall","Battle of the Twenty-first Hall - Raid","Weekly");
--	self:AddInstance("The Icy Crevasse","The Icy Crevasse - Raid","Weekly");
--	self:AddInstance("Attack At Dawn","Attack At Dawn - Raid","Weekly");


	self:AddInstance("The Tower of Orthanc: Fire/Frost",{},"Weekly");
	self:AddInstance("The Tower of Orthanc: Lightning",{},"Weekly");
	self:AddInstance("The Tower of Orthanc: Acid",{},"Weekly");
	self:AddInstance("The Tower of Orthanc: Shadow",{},"Weekly");
	self:AddInstance("The Tower of Orthanc: Saruman",{},"Weekly");
	self:AddInstance("The Foundry",{"Challenge","Ring-forge Falls"},"Bi-Weekly");
	self:AddInstance("Dargn\195\161kh Unleased",{"Challenge","Darg","Unleashed"},"Bi-Weekly");
	self:AddInstance("Pits of Isengard",{"Challenge","Pits","Isengard"},"Bi-Weekly");
	self:AddInstance("Fangorn's Edge",{"Challenge","Fangorn's Savior"},"Bi-Weekly");
	self:AddInstance("Draigoch's Lair",{},"Weekly");
	self:AddInstance("Helegrod: Thorog",{"Helegrod Drugoth","Raid"},"Weekly");
	self:AddInstance("Helegrod: Drake Wing",{"Helegrod Drake","Raid"},"Weekly");--
	self:AddInstance("Helegrod: Giant Wing",{"Helegrod Giant","Raid"},"Weekly");--
	self:AddInstance("Helegrod: Spider Wing",{"Helegrod Spider","Raid"},"Weekly");--
	self:AddInstance("Ford of Bruinen",{"Ford","Bruinen","Raid"},"Weekly");--
	self:AddInstance("Siege of Gondamon",{"Siege","Gondamon","Raid"},"Weekly");--
	self:AddInstance("Trouble in Tuckborough",{"Trouble","Tuckborough","Raid"},"Weekly");--
	self:AddInstance("Stand at Amon S\195\186l",{"Stand","Amon","Raid"},"Weekly");--
	self:AddInstance("Strike Against Dannenglor",{"Strike Against Dannenglor","Raid"},"Weekly");--
	self:AddInstance("Protectors of Thang\195\186lhad",{"Protectors","Thang","Raid"},"Weekly");--
	self:AddInstance("Breaching the Necromancer's Gate",{"Breaching","Necromancer's Gate","Raid"},"Weekly");--
	self:AddInstance("Assault on the Ringwraiths' Lair",{"Assault","Ringwraiths' Lair","Raid"},"Weekly");--
	self:AddInstance("The Battle in the Tower",{"The Battle","Tower","Raid"},"Weekly");--
	self:AddInstance("Survival: Barrow-Downs",{"Survival","Barrow-downs","Raid"},"Weekly");--
	self:AddInstance("Defence of the Prancing Pony",{"Defence","Prancing Pony","Raid"},"Weekly");--
	self:AddInstance("Thievery and Mischief",{"Thievery","Mischief","Raid"},"Weekly");--
	self:AddInstance("Rescue in N\195\186rz Gh\195\161shu",{"Rescue","rz Gh","shu","Raid"},"Weekly");
	self:AddInstance("Battle of the Deep-way",{"Battle","Deep-way","Raid"},"Weekly");
	self:AddInstance("Battle of the Way of Smiths",{"Battle","Way of Smiths","Raid"},"Weekly");
	self:AddInstance("Battle of the Twenty-first Hall",{"Battle","Twenty-first Hall","Raid"},"Weekly");
	self:AddInstance("The Icy Crevasse",{"Icy","Crevasse","Raid"},"Weekly");
	self:AddInstance("Attack At Dawn",{"Attack","Dawn","Raid"},"Weekly");	

	
	self:ResetLocks();
end

function Settings:AddInstance(name,quest,reset)
	self.settingsTable.instances[self.settingsTable.instances.length] = name;
	self.settingsTable.instances.length = self.settingsTable.instances.length + 1;
	self.settingsTable.instanceQuestMap[quest] = name;
	self.settingsTable.resetSchedule[name] = reset;
end	

function Settings:AddPlayer(player)
	for i=0, self.settingsTable.players.length-1 do
		if(self.settingsTable.players[i] == player) then
			return
		end
	end
	self.settingsTable.players[self.settingsTable.players.length] = player; self.settingsTable.players.length = self.settingsTable.players.length + 1;
	for i=0, self.settingsTable.instances.length-1 do
		local instance = self.settingsTable.instances[i];
		if(self.settingsTable.locks[instance] == nil) then
			self.settingsTable.locks[instance] = {};
		end
		self.settingsTable.locks[instance][player] = false;
	end
	self:Save();
end

function Settings:ResetLocks()
	for i=0, self.settingsTable.instances.length-1 do
		local instance = self.settingsTable.instances[i];
		for j=0, self.settingsTable.players.length-1 do
			local player = self.settingsTable.players[j];
			if(self.settingsTable.locks[instance] == nil) then
				self.settingsTable.locks[instance] = {};
			end
			self.settingsTable.locks[instance][player] = false;
		end
	end
	
	self:GetNewWeeklyResetTime();
	self:GetNewBiWeeklyResetTime();
end

function Settings:GetNewWeeklyResetTime()
	local date = Turbine.Engine.GetDate();
	self.settingsTable.weeklyResetTime = {};
	self.settingsTable.weeklyResetTime.Hour = date.Hour;
	self.settingsTable.weeklyResetTime.DayOfWeek = date.DayOfWeek;
	self.settingsTable.weeklyResetTime.DayOfYear = date.DayOfYear;
	self.settingsTable.weeklyResetTime.Year = date.Year;
	self:Save();
end

function Settings:GetNewBiWeeklyResetTime()
	local date = Turbine.Engine.GetDate();
	self.settingsTable.biWeeklyResetTime = {};
	self.settingsTable.biWeeklyResetTime.Hour = date.Hour;
	self.settingsTable.biWeeklyResetTime.DayOfWeek = date.DayOfWeek;
	self.settingsTable.biWeeklyResetTime.DayOfYear = date.DayOfYear;
	self.settingsTable.biWeeklyResetTime.Year = date.Year;
	self:Save();
end

function Settings:Save()
	Turbine.PluginData.Save(Turbine.DataScope.Account, "ChallengeTracker", self.settingsTable);
end


function Settings:AddLock(player,instance)
	if(self.settingsTable.locks[instance] == nil) then
		self.settingsTable.locks[instance] = {};
	end
	Turbine.Shell.WriteLine("Adding lock for "..instance.." for "..player);
	self.settingsTable.locks[instance][player] = true;
	self:Save();
end

function Settings:RemoveLock(player,instance)
	if(self.settingsTable.locks[instance] == nil) then
		self.settingsTable.locks[instance] = {};
	end
	Turbine.Shell.WriteLine("Removing lock for "..instance.." for "..player);
	self.settingsTable.locks[instance][player] = false;
	self:Save();
end

function Settings:ResetWeekly()
	Turbine.Shell.WriteLine("Resetting Weekly Locks");
	self:ResetLocks();
end

function Settings:ResetBiWeekly()
	Turbine.Shell.WriteLine("Resetting Bi-Weekly Locks");
	for i=0, self.settingsTable.instances.length-1 do
		local instance = self.settingsTable.instances[i];
		if(self.settingsTable.resetSchedule[instance] == "Bi-Weekly") then
			for j=0, self.settingsTable.players.length-1 do
				local player = self.settingsTable.players[j];
				if(self.settingsTable.locks[instance] == nil) then
					self.settingsTable.locks[instance] = {};
				end
				self.settingsTable.locks[instance][player] = false;
			end
		end
	end
	
	self:GetNewBiWeeklyResetTime();
end

function Settings:Update(message)
	for questparts,name in pairs(self.settingsTable.instanceQuestMap) do
		local found = (#questparts > 0);
		for junk,questpart in pairs(questparts) do
			if(string.find(message,questpart) == nil) then
				found=false;
				break;
			end
		end
		if(found==true) then
			self:AddLock(self.currentPlayer,name);
			return true;
		end
	end
	return false;
end