import "Turbine.UI";
import "ChallengeTracker.ChallengeTrackerMain.Settings";

CheckBoxItem = class(Turbine.UI.TreeNode);

globalWindow = nil;

function CheckBoxItem:Constructor(name,value)
	Turbine.UI.TreeNode.Constructor(self);
	self:SetSize(450,20);

	self.checkbox = Turbine.UI.Lotro.CheckBox();
	self.checkbox:SetChecked(value);
	self.checkbox:SetPosition(0,0);
	self.checkbox:SetSize(20,20);
	self.checkbox:SetParent(self);
	self.checkbox:SetZOrder(1);
	self.checkbox:SetEnabled(false);
	self.checkbox:SetWantsKeyEvents(true);
	
	self.baseText = name;

	self.text = Turbine.UI.Lotro.TextBox();
	self.text:SetSize(430,20);
	self.text:SetPosition(20,0);
	self.text:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
	self.text:SetParent(self);
	self.text:SetBackColor(Turbine.UI.Color(1,0,0));
	self.text:SetForeColor(Turbine.UI.Color(1,1,1));
	self.text:SetText("  "..self.baseText);
	self.text:SetSelectable(false);
	self.text:SetReadOnly(true);
end

function CheckBoxItem:Init()
	self.text.MouseClick = function(sender, args)
		self.checkbox:SetEnabled(true);
		self.checkbox:SetChecked(not self.checkbox:IsChecked());
		self.checkbox:SetEnabled(false);
		if(self.checkbox:IsChecked()) then
			globalWindow.settings:AddLock(globalWindow.settings.currentPlayer,self.baseText);
		else
			globalWindow.settings:RemoveLock(globalWindow.settings.currentPlayer,self.baseText);
		end
	end 
end

TreeHeader = class(Turbine.UI.TreeNode);

function TreeHeader:Constructor(name)
	Turbine.UI.TreeNode.Constructor(self);
	self:SetSize(450,20);

	self.baseText = name;
	
	self.text = Turbine.UI.Lotro.TextBox();
	self.text:SetSize(450,20);
	self.text:SetPosition(0,0);
	self.text:SetParent(self);
	self.text:SetFont(Turbine.UI.Lotro.Font.TrajanPro14);
	self.text:SetBackColor(Turbine.UI.Color(1,0,0));
	self.text:SetForeColor(Turbine.UI.Color(1,1,1));
	self.text:SetText("+"..self.baseText);
	self.text:SetSelectable(false);
	self.text:SetReadOnly(true);
	
	self.text.MouseClick = function(sender,args)
		if(self:IsExpanded()) then
			self.text:SetText("-"..self.baseText);
		else
			self.text:SetText("+"..self.baseText);
		end
	end
	
end

TreeItem = class(Turbine.UI.TreeNode);

function TreeItem :Constructor(name)
	Turbine.UI.TreeNode.Constructor(self);
	self:SetBackColor(Turbine.UI.Color(.2,.2,.2));
	self:SetSize(450,15);

	self.text = Turbine.UI.TextBox();
	self.text:SetSize(450,15);
	self.text:SetPosition(0,0);
	self.text:SetFont(Turbine.UI.Lotro.Font.TrajanPro13);
	self.text:SetParent(self);
	self.text:SetForeColor(Turbine.UI.Color(.1,1,1));
	self.text:SetText("  "..name);
	self.text:SetSelectable(false);
	self.text:SetReadOnly(true);
end

TrackerWindow = class(Turbine.UI.Window);

function TrackerWindow:Constructor(settings)
	Turbine.UI.Lotro.Window.Constructor(self);
	globalWindow = self;
	self.settings = settings;
	self:SetSize(500,510);
	self:SetPosition(0,0);
	self:SetText("Challenge Tracker");
	self:SetVisible(true);

	self.instancesButton = Turbine.UI.Lotro.Button();
	self.instancesButton:SetText("Sort by Instance");
	self.instancesButton:SetParent(self);
	self.instancesButton:SetPosition(26,50);
	self.instancesButton:SetSize(132,20);
	self.instancesButton.Click = function(sender,args) self:displayInstancesTab(); end

	self.playersButton = Turbine.UI.Lotro.Button();
	self.playersButton:SetText("Sort by Character");
	self.playersButton:SetParent(self);
	self.playersButton:SetPosition(184,50);
	self.playersButton:SetSize(132,20);
	self.playersButton.Click = function(sender,args) self:displayPlayersTab(); end

	self.markButton = Turbine.UI.Lotro.Button();
	self.markButton:SetText("Mark Progress");
	self.markButton:SetParent(self);
	self.markButton:SetPosition(342,50);
	self.markButton:SetSize(132,20);
	self.markButton.Click = function(sender,args) self:displayMarkTab(); end
	
	self.mainWindow = Turbine.UI.TreeView();
	self.mainWindow:SetParent(self);
	self.mainWindow:SetPosition(25,80);
	self.mainWindow:SetSize(450,390);

	self.mainWindowScroll = Turbine.UI.Lotro.ScrollBar();
	self.mainWindowScroll:SetOrientation(Turbine.UI.Orientation.Vertical);
	self.mainWindowScroll:SetParent(self);
	self.mainWindowScroll:SetPosition(480,80);
	self.mainWindowScroll:SetSize(10,360);
	self.mainWindow:SetVerticalScrollBar(self.mainWindowScroll);
	
	self.resetBiWeekly = Turbine.UI.Lotro.Button();
	self.resetBiWeekly:SetText("Manual Reset Bi-Weekly");
	self.resetBiWeekly:SetParent(self);
	self.resetBiWeekly:SetPosition(56,480);
	self.resetBiWeekly:SetSize(166,20);
	self.resetBiWeekly.MouseDoubleClick = function (sender,args) 
		self.settings:ResetBiWeekly();
		self:refreshCurrentTab();
	end
	
	self.resetWeekly = Turbine.UI.Lotro.Button();
	self.resetWeekly:SetText("Manual Reset Weekly");
	self.resetWeekly:SetParent(self);
	self.resetWeekly:SetPosition(278,480);
	self.resetWeekly:SetSize(166,20);
	self.resetWeekly.MouseDoubleClick = function (sender,args) 
		self.settings:ResetWeekly();
		self:refreshCurrentTab(); 
	end
				
	self:displayInstancesTab();
end

--save state?
function TrackerWindow:refreshCurrentTab()
	if(self.currentTab == "instances") then self:displayInstancesTab();
	elseif(self.currentTab == "players") then self:displayPlayersTab();
	elseif(self.currentTab == "mark") then self:displayMarkTab(); end
end

function TrackerWindow:displayInstancesTab()
	self.currentTab = "instances";
	
	local nodes = self.mainWindow:GetNodes();
	nodes:Clear();
	local instanceTable = self.settings.settingsTable.instances;
	local playerTable = self.settings.settingsTable.players;
	local locksTable = self.settings.settingsTable.locks;

	for i=0, instanceTable.length-1 do
		local instanceName = instanceTable[i];
		local header = ChallengeTracker.ChallengeTrackerMain.TreeHeader(instanceName);
		local headerChildren = header:GetChildNodes();
		nodes:Add(header);
		for j=0, playerTable.length-1 do
			local playerName = playerTable[j];
			if(locksTable[instanceName][playerName]) then
				local item = ChallengeTracker.ChallengeTrackerMain.TreeItem(playerName);
				headerChildren:Add(item);
			end
		end
	end
end

function TrackerWindow:displayPlayersTab()
	self.currentTab = "players";
	
	local nodes = self.mainWindow:GetNodes();
	nodes:Clear();
	local instanceTable = self.settings.settingsTable.instances;
	local playerTable = self.settings.settingsTable.players;
	local locksTable = self.settings.settingsTable.locks;

	for j=0, playerTable.length-1 do
		local playerName = playerTable[j];
		local header = ChallengeTracker.ChallengeTrackerMain.TreeHeader(playerName);
		local headerChildren = header:GetChildNodes();
		nodes:Add(header);
		for i=0, instanceTable.length-1 do
			local instanceName = instanceTable[i];
			if(locksTable[instanceName][playerName]) then
				local item = ChallengeTracker.ChallengeTrackerMain.TreeItem(instanceName);
				headerChildren:Add(item);
			end
		end
	end
end

function TrackerWindow:displayMarkTab()
	self.currentTab = "mark";

	local nodes = self.mainWindow:GetNodes();
	nodes:Clear();
	local instanceTable = self.settings.settingsTable.instances;
	local playerName = self.settings.currentPlayer;
	local locksTable = self.settings.settingsTable.locks;

	for i=0, instanceTable.length-1 do
		local instanceName = instanceTable[i];
		local value = locksTable[instanceName][playerName];
		local header = ChallengeTracker.ChallengeTrackerMain.CheckBoxItem(instanceName,value);
		nodes:Add(header);
		header:Init();
	end
end
	